/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "houdini/houdini.pmh"

namespace fe
{

const GEO_AttributeOwner	SurfaceAccessorHoudini::ms_elementTable[]=
{
	GEO_POINT_DICT,		// e_point
	GEO_VERTEX_DICT,	// e_vertex
	GEO_PRIMITIVE_DICT,	// e_primitive
	GEO_DETAIL_DICT		// e_detail
};

void SurfaceAccessorHoudini::addInteger(void)
{
//	feLog("SurfaceAccessorHoudini::addInteger \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(I32),GB_ATTRIB_INT,NULL);
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addInteger"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addReal(void)
{
//	feLog("SurfaceAccessorHoudini::addReal \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		const Real zero=0.0;
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(Real),GB_ATTRIB_FLOAT,&zero);
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addReal"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

void SurfaceAccessorHoudini::addSpatialVector(void)
{
//	feLog("SurfaceAccessorHoudini::addSpatialVector \"%s\" %p\n",
//			m_name.c_str(),m_pGdpChangeable);

	if(m_pGdpChangeable)
	{
		if(m_element==SurfaceAccessibleI::e_primitive)
		{
			m_pGdpChangeable->addPrimAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
		}
		else if(m_element==SurfaceAccessibleI::e_point)
		{
			m_pGdpChangeable->addPointAttrib(m_name.c_str(),
					sizeof(UT_Vector3),GB_ATTRIB_VECTOR,NULL);
		}
		bindInternal(m_element,m_name);
	}
	if(!m_handle.isAttributeValid())
	{
		feLog("SurfaceAccessorHoudini::addSpatialVector"
			" could not add attribute \"%s\"\n",m_name.c_str());
		return;
	}
}

} // namespace
