/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __grass_Blade__
#define __grass_Blade__

namespace fe
{

/**************************************************************************//**
	@brief minimal semi-implicit curve sim

	@ingroup grass
*//***************************************************************************/
class FE_DL_EXPORT Blade
{
	public:
		typedef enum
		{
			e_boundingSphere=0x00,
			e_nearestPoint=0x01,
			e_rayCastOneStep=0x02,
			e_rayCastHalvsies=0x03
		} CollisionMethod;

						Blade(void);
virtual					~Blade(void)										{}

		void			setTime(Real a_t)			{ m_t=a_t; }
		Real			time(void) const			{ return m_t; }

		void			setTimeStep(Real a_h)		{ m_h=a_h; }
		Real			timeStep(void) const		{ return m_h; }

		void			setTension(Real a_k)		{ m_k=a_k; }
		Real			tension(void) const			{ return m_k; }

		void			setRestoration(Real a_restoration)
						{	m_restoration=a_restoration; }
		Real			restoration(void) const
						{	return m_restoration; }

		void			setDrag(Real a_c)			{ m_c=a_c; }
		Real			drag(void) const			{ return m_c; }

		void			setBend(Real a_bend)		{ m_bend=a_bend; }
		Real			bend(void) const			{ return m_bend; }

		void			setGrabBias(Real a_grabBias){ m_grabBias=a_grabBias; }
		Real			grabBias(void) const		{ return m_grabBias; }

		void			setCollisionGap(Real a_collisionGap)
						{	m_collisionGap=a_collisionGap; }
		Real			collisionGap(void) const
						{	return m_collisionGap; }

		void			setMass(Real a_m)			{ m_m=a_m; }
		Real			mass(void) const			{ return m_m; }

		void			setWind(Real a_wind)
						{	m_wind=a_wind; }
		Real			wind(void) const
						{	return m_wind; }

		void			setTurbulence(Real a_turbulence)
						{	m_turbulence=a_turbulence; }
		Real			turbulence(void) const
						{	return m_turbulence; }

		void			setCollisionMethod(CollisionMethod a_method)
						{	m_method=a_method; }
		CollisionMethod	collisionMethod(void)
						{	return m_method; }

		void			setCollider(sp<SurfaceI> a_spSurfaceI)
						{	m_spCollider=a_spSurfaceI; }
		sp<SurfaceI>	collider(void)
						{	return m_spCollider; }

		void			setDrawI(sp<DrawI> a_spDrawI)
						{	m_spDrawI=a_spDrawI; }

		void			storeAsRest(U32 a_index);

		void			populate(void);
		void			step(U32 a_index);

		std::vector<SpatialVector>&	tangent(void)	{ return m_tangent; }
		std::vector<SpatialVector>&	facing(void)	{ return m_facing; }
		std::vector<SpatialVector>&	location(void)	{ return m_location; }
		std::vector<Real>&	velocity(void)			{ return m_velocity; }

	private:
	class Contact
	{
		public:
							Contact(void)	{ clear(); }
							~Contact(void)									{}

			void			clear(void)
							{
								set(m_offset);
								set(m_normal);
								m_active=FALSE;
							}
			void			setOffset(SpatialVector a_offset)
							{
								m_offset=a_offset;
								m_active=TRUE;
							}
			void			setNormal(SpatialVector a_normal)
							{
								m_normal=a_normal;
							}

			BWORD			active(void)	{ return m_active; }
			SpatialVector	offset(void)	{ return m_offset; }
			SpatialVector	normal(void)	{ return m_normal; }

		private:
			SpatialVector	m_offset;
			SpatialVector	m_normal;
			BWORD			m_active;
	};

		void			computeAngles(U32 a_index, Vector<4,Real>& a_rAngle,
								const SpatialVector* a_pDelta);

		std::vector<Contact>			m_contact;
		std::vector<SpatialVector>		m_tangent;		// root direction
		std::vector<SpatialVector>		m_facing;		// unit facing
		std::vector<SpatialVector>		m_restFacing;	// original facing
		std::vector<SpatialVector>		m_location;		// point locations
		std::vector<Real>				m_velocity;		// angular velocities
		std::vector<Real>				m_restLength;	// full base to tip
		std::vector<Real>				m_tangentBias;	// tend to keep same dir
		std::vector< Vector<4,Real> >	m_length;		// segment length
		std::vector< Vector<4,Real> >	m_rest;			// rest angle

		Real				m_t;
		Real				m_k;
		Real				m_restoration;
		Real				m_c;
		Real				m_bend;
		Real				m_grabBias;
		Real				m_collisionGap;
		Real				m_m;
		Real				m_h;
		Real				m_wind;
		Real				m_turbulence;
		CollisionMethod		m_method;

		sp<SurfaceI>		m_spCollider;
		sp<DrawI>			m_spDrawI;

		Matrix<4,4,Real>	m_invA;
		Matrix<4,4,Real>	m_dfdx;
		Matrix<4,4,Real>	m_hdfdx;
};

} /* namespace */

#endif /* __grass_Blade__ */

